{% comment %}
# SPDX-License-Identifier: AGPL-3.0-or-later
{% endcomment %}

{% load i18n %}

{% block toolbar %}

  <section class="toolbar">
    <div class="btn-toolbar" role="toolbar">

      {% include "clients-button.html" with clients=app_info.clients enabled=is_enabled %}

      {% if app_info.donation_url %}
        <a target="_blank" rel="noopener noreferrer"
           class="btn btn-default button-secondary button-donate" role="button"
           href="{{ app_info.donation_url }}">
          <span class="fa fa-heart"></span>
          {% trans "Donate" %}
          <span class="fa fa-external-link"></span>
        </a>
      {% endif %}

      {% if has_diagnostics %}
        <!-- Single button -->
        <div class="btn-group button-secondary">
          <button type="button" class="btn btn-default dropdown-toggle"
                  data-toggle="dropdown" aria-haspopup="true"
                  aria-expanded="false" id="id_extra_actions_button">
            <span class="fa fa-cog" aria-hidden="true"></span>
          </button>
          <div class="dropdown-menu" aria-labelledby="id_extra_actions_button">
            {% if has_diagnostics %}
              {% include "diagnostics_button.html" with app_id=app_id enabled=is_enabled %}
            {% endif %}
          </div>
        </div>
      {% endif %}

    </div>
    <div class="toolbar-collapsed-content">
      {% include "clients.html" with clients=app_info.clients enabled=is_enabled %}
    </div>
  </section>

{% endblock toolbar %}
