{% extends "app.html" %}
{% comment %}
# SPDX-License-Identifier: AGPL-3.0-or-later
{% endcomment %}
{% load bootstrap %}
{% load i18n %}

{% block status %}
  {{ block.super }}

  <h3>{% trans "Access" %}</h3>

  <p>
    {% blocktrans trimmed %}
        Cockpit will only work when accessed using the following URLs.
    {% endblocktrans %}
  </p>

  <ol>
    {% for url_ in urls %}
    <li>{{ url_ }}</li>
    {% endfor %}
  </ol>
{% endblock %}
