{% extends "app.html" %}
{% comment %}
# SPDX-License-Identifier: AGPL-3.0-or-later
{% endcomment %}

{% load bootstrap %}
{% load i18n %}
{% load static %}

{% block description %}

  {% for paragraph in description %}
    <p>{{ paragraph|safe }}</p>
  {% endfor %}

  <p>
    {% url 'config:index' as index_url %}
    {% if domainname %}
      {% blocktrans trimmed with domainname=domainname %}
        Your XMPP server domain is set to <b>{{ domainname }}</b>. User
        IDs will look like <i>username@{{ domainname }}</i>. You
        can setup your domain on the system
        <a href="{{ index_url }}">Configure</a> page.
      {% endblocktrans %}
    {% else %}
      Your XMPP server domain is not set. You can setup your domain on
      the system <a href="{{ index_url }}">Configure</a> page.
    {% endif %}
  </p>

{% endblock %}

{% block page_js %}
  <script type="text/javascript"
          src="{% static 'ejabberd/ejabberd.js' %}"></script>
{% endblock %}
